; #########################################################################

  ; ************************************************
  ; This example was written by Raymond Filiatreault
  ; ************************************************
  ;
  ; The purpose of this example is to show the utility of Mixed Integers
  ; for the addition of a series of fractions.
  ;
  ; The chosen example is the calculation of the value of the Napierian
  ; constant "e" which is the base of the natural logarithms. That
  ; constant is 2.718281828459045.... which can be calculated according
  ; to the following infinite series of factorial fractions:
  ;
  ; e = Sum(1/n!), n going from 0 to infinity (by convention, 0! = 1).
  ;
  ; If calculations were performed strictly per the equation using the
  ; lower 16 bits of a dword for the fractional portion, the result
  ; would have 5 decimals places. Since, in this case, the expected
  ; result is less than 3, multiplying every fraction by 10000 would
  ; produce a result 10000 times higher which would be less than 30000
  ; and within the given limits of Mixed Integers. By adjusting that
  ; result accordingly, the final answer can now have 9 decimal places.
  ;
  ; If you run the program, you will notice that the result is accurate to
  ; the 8th decimal place, which is then only in error by 1.
  ;
  ; N.B.    No checking for the error code is performed in this example
  ;         since all necessary precautions are taken to avoid an
  ;         overflow of the limits.
  ;
  ; --------------------------------------------------

    .386
    .model flat, stdcall  ; 32 bit memory model
    option casemap :none  ; case sensitive

; #########################################################################

      include \masm32\include\windows.inc
      include \masm32\include\user32.inc
      include \masm32\include\kernel32.inc
      include \masm32\include\Mix.inc

      includelib \masm32\lib\user32.lib
      includelib \masm32\lib\kernel32.lib
      includelib \masm32\lib\Mix.lib

; #########################################################################

    .data

    sums        dd  0           ;accumulator for the sum of fractions
    multiplier  dd  10000       ;multiplyer of each fraction
    szBuffer    db  16 DUP(0)   ;buffer to receive the converted result
    szTitle     db  "Napierian Base",0

    .code

start:
    invoke MixFromInt,multiplier ;transform the multiplier to Mixed Integer

    mov sums,eax        ;initialize the accumulator with it since it would be 
                        ;divided by 1 (0!) in the first term of the series
 
    mov edi,1           ;edi will hold the current factorial (0! to start)
    xor esi,esi         ;esi will hold the current number (0 to start)

  .while eax != 0       ;this loop is terminated if nothing is being added
                        ;to the accumulator

    inc esi             ;advance to the next number
    mov eax,edi         ;copy current factorial to eax
    mul esi             ;get next factorial
    mov edi,eax         ;copy it back as current factorial
    invoke MixDiv,multiplier,eax    ;the MixDiv function allows the use of
                                    ;two integers as parameters and returns
                                    ;a Mixed Integer number
    add sums,eax        ;add this result to the accumulator

  .endw

; At this point, the obtained result should be divided by the multiplier
; if it needed to be used for other calculations which could exceed the
; limits of Mixed Integers. In this example, it will simply be converted
; as is to its decimal representation and the decimal point shifted to its
; proper location before displaying it.

    invoke MixToA,sums,ADDR szBuffer    ;convert the result in the
                                        ;accumulator to alphnumeric
    mov edi,offset szBuffer+1           ;position of the 2nd digit
    mov eax,[edi]                       ;get the 2nd to 5th digits
    mov [edi+1],eax                     ;copy them one position to the right
    mov byte ptr [edi],"."              ;put the decimal point where it belongs

    invoke MessageBox,0,ADDR szBuffer,ADDR szTitle,MB_OK    ;display result
    invoke ExitProcess,0
    
end start

