; #########################################################################

  ; --------------------------------------------------
  ; This procedure was written by Raymond Filiatreault
  ; --------------------------------------------------

    .386
    .model flat, stdcall  ; 32 bit memory model
    option casemap :none  ; case sensitive

      include \masm32\include\Mix.inc

      includelib \masm32\lib\Mix.lib

    .code

; #########################################################################

MixArccosh proc public MInt:DWORD
        
    ; -------------------------------------------------------------
    ; Computes the hyperbolic arccosine of a number per the following
    ; relationship:
    ;
    ;           arccosh(x) = ln( x + (x^2 - 1)^.5)
    ;
    ; eax, ecx, edx used
    ;
    ; MInt:    The number in a Mixed Integer format
    ;
    ; EXAMPLE:
    ;
    ;       invoke MixArccosh, var1 
    ;
    ; The result is returned in eax in a Mixed Integer format. If the
    ; parameter is the error code 80000000h, it is returned unchanged.
    ;
    ; The error code will also be returned if the absolute value of the
    ; parameter is lower than 1.
    ;
    ; Although the cosh of a number can only be positive for the entire
    ; range of positive and negative numbers, positive and negative
    ; parameters are accepted for this procedure. The sign is transfered
    ; to the returned value.
    ;
    ; -------------------------------------------------------------

    mov eax,MInt
    .if eax == ERROR
        ret
    .endif

    or  eax,eax
    jns @F
    neg eax             ;get the absolute value of the parameter
 @@:
    cmp eax,1 shl 16
    jae @F
    mov eax,ERROR
    ret
    
 @@:
    push edi
    mov edi,eax         ;keep the absolute value of the parameter
    mul eax
    dec edx
    invoke sqrt64,edx,eax
    add eax,edi
    shr eax,1           ;eax/2 (precaution against high bit being set)
    invoke MixLNx,eax   ;ln(eax)-ln(2)
    add eax,MixLN2      ;add back the ln(2)
    mov ecx,MInt
    or  ecx,ecx         ;get the sign of the parameter
    jns @F
    neg eax
  @@:
    pop edi
    ret

MixArccosh endp

; #########################################################################

end

