; #########################################################################

  ; --------------------------------------------------
  ; This procedure was written by Raymond Filiatreault
  ; --------------------------------------------------

    .386
    .model flat, stdcall  ; 32 bit memory model
    option casemap :none  ; case sensitive

      include \masm32\include\Mix.inc

      includelib \masm32\lib\Mix.lib

    .code

; #########################################################################

MixArcsin proc public MInt:DWORD, qFlag:DWORD
        
    ; -------------------------------------------------------------
    ; Computes the arcsine of a fraction by first computing cos(x)
    ;
    ;           cos(x) = (1-sin(x)^2)^.5    (i.e. sin(x)^2 + cos(x)^2 = 1)
    ;
    ; and then invoking the MixArctan procedure based on the relationship
    ;
    ;           tan(x) = sin(x)/cos(x)
    ;
    ; eax, ecx, edx used
    ;
    ; MInt:     The value of the sine in a Mixed Integer format
    ;
    ; qFlag:    0 for the 270-360/0-90 quadrants,
    ;           !0 for the 90-180/180-270 quadrants
    ;
    ; EXAMPLE:
    ;
    ;       invoke MixArcsin, var1, 0 
    ;
    ; The result is returned in EAX in a Mixed Integer format. If the first
    ; parameter is the error code 80000000h, it is returned in EAX.
    ;
    ; The value provided by the first (MInt) parameter can be positive
    ; or negative but, if its absolute value exceeds 1.00000, the error
    ; code is returned in EAX.
    ;
    ; -------------------------------------------------------------

    mov eax,MInt
    .if eax == ERROR
        ret
    .endif

    or  eax,eax
    jnz @F
                                ;if the sine = 0, the angle = 0 or 180
    .if qFlag != 0
        mov eax,180 shl 16
    .endif
    ret

  @@:
    jns @F
    neg eax

  @@:
    cmp eax,10000h
    jbe @F
    mov eax,ERROR
    ret

  @@:
    jnz @F                      ;if the abs(sine) = 1, the angle = 90 or 270
    .if MInt < ERROR            ;positive
        mov eax,90 shl 16
    .else                       ;negative
        mov eax,270 shl 16
    .endif
    ret

  @@:
    mul eax                     ;sin(x)^2
    neg eax                     ;1-sin(x)^2 = cos(x)^2
    invoke sqrt32,eax           ;cos(x)
    .if qFlag != 0
        neg eax
    .endif
    invoke MixArctan,MInt,eax

    ret
    
MixArcsin endp

; #########################################################################

end

