; #########################################################################

  ; --------------------------------------------------
  ; This procedure was written by Raymond Filiatreault
  ; --------------------------------------------------

    .386
    .model flat, stdcall  ; 32 bit memory model
    option casemap :none  ; case sensitive

      include \masm32\include\Mix.inc

      includelib \masm32\lib\Mix.lib

    .code

; #########################################################################

MixArcsinh proc public MInt:DWORD
        
    ; -------------------------------------------------------------
    ; Computes the hyperbolic arcsine of a number per the following
    ; relationship:
    ;
    ;           arcsinh(x) = ln( x + (x^2 + 1)^.5)
    ;
    ; eax, ecx, edx used
    ;
    ; MInt:    The number in a Mixed Integer format
    ;
    ; EXAMPLE:
    ;
    ;       invoke MixArcsinh, var1 
    ;
    ; The result is returned in eax in a Mixed Integer format. If the
    ; parameter is the error code 80000000h, it is returned unchanged.
    ;
    ; The parameter can be positive or negative. Because computing with
    ; a negative number could result in taking the logarithm of a very
    ; small inaccurate number, computing is done with the absolute value
    ; of the parameter and the negative result reported for negative
    ; parameters.
    ;
    ; -------------------------------------------------------------

    mov eax,MInt
    .if eax == ERROR
        ret
    .endif

    imul eax
    inc edx
    invoke sqrt64,edx,eax
    mov ecx,MInt
    or  ecx,ecx
    jns @F
    neg ecx
  @@:
    add eax,ecx
    shr eax,1           ;eax/2 (precaution against high bit being set)
    invoke MixLNx,eax   ;ln(eax)-ln(2)
    add eax,MixLN2      ;add back the ln(2)
    mov ecx,MInt
    or  ecx,ecx         ;get the sign of the parameter
    jns @F
    neg eax
  @@:
    ret

MixArcsinh endp

; #########################################################################

end

