; #########################################################################

  ; --------------------------------------------------
  ; This procedure was written by Raymond Filiatreault
  ; --------------------------------------------------

    .386
    .model flat, stdcall  ; 32 bit memory model
    option casemap :none  ; case sensitive

      include \masm32\include\Mix.inc

    .code

; #########################################################################

MixFromA proc public lpNum:DWORD
        
    ; -------------------------------------------------------------
    ; Converts the decimal representation of a number from a
    ; 0 terminated ANSI string to a Mixed Integer format.
    ;
    ; eax, ecx, edx used
    ;
    ; lpNum: Address where the 0 terminated string is located.
    ; 
    ; EXAMPLE:
    ;
    ;       invoke MixFromA, ADDR szDecimalNum
    ;
    ; Leading spaces in the string are disregarded. Then, only one 
    ; "-" or "=" sign and only one decimal point is allowed. All other
    ; characters must be decimal digits until the terminating 0.
    ;
    ; The result is returned in eax.
    ;
    ; The returned result will be 80000000h (-0) if
    ;       the absolute value of the integer portion exceeds 7FFFh,
    ; or    an unacceptable character is noticed in the input string,
    ; or    the input string is empty.
    ;
    ; The number of digits between the decimal point and the terminating 0
    ; in the input string has no limit. However, only the first 8 would
    ; be used since subsequent ones would not have any effect on the result.
    ;
    ; -------------------------------------------------------------

    push ebx
    push esi
    push edi

    xor edi,edi             ;will be used for the integer portion
    xor ecx,ecx             ;will be used for the decimal portion
    xor ebx,ebx             ;will be used for the sign flag and as a counter
    xor eax,eax
    
    mov esi,lpNum
  @@:
    lodsb
    cmp al," "
    jz  @B                  ;disregard leading spaces
    or  al,al               ;empty string?
    jnz @F

srcerror:
    mov eax,ERROR
    jmp eXit

  @@:
    cmp al,"+"
    jz  nextchk

    cmp al,"-"
    jnz @F
    mov ebx,80000000h       ;sign code in bit 31 of ebx

nextchk:
    lodsb
    or  al,al               ;empty string?
    jz  srcerror

  @@:
    cmp al,"."
    jz  decimal
    sub al,"0"
    jc  srcerror
    cmp al,9
    ja  srcerror
    lea edi,[edi+edi*4]
    lea edi,[edi*2+eax]     ;edi*10 + new digit
    cmp edi,32767           ;check after each digit to prevent the
                            ;possibility of a long string returning
                            ;an unacceptable number
    ja  srcerror
    lodsb
    or  al,al
    jnz @B

nodecimal:
    mov eax,edi
    shl eax,16
    jmp cleanup

decimal:
    lodsb
    or  al,al
    jz  convert
    sub al,"0"
    jc  srcerror
    cmp al,9
    ja  srcerror
    lea ecx,[ecx+ecx*4]
    lea ecx,[ecx*2+eax]
    inc bl                  ;count of digits after decimal point
    cmp bl,8
    jb  decimal
    jmp convert1

convert:
    or  bl,bl
    jz  nodecimal
  @@:
    lea ecx,[ecx+ecx*4]
    inc bl
    add ecx,ecx
    cmp bl,8
    jb @B
  
convert1:
    mov eax,20000h
    mul ecx
    mov ecx,100000000
    div ecx
    shr eax,1
    jnc @F
    inc eax
  @@:
    shl edi,16
    add eax,edi
    jns cleanup
    dec eax

cleanup:
    or  ebx,ebx
    jns eXit
    neg eax
    
eXit:
    pop edi
    pop esi
    pop ebx
    ret
    
MixFromA endp

; #########################################################################

end
