; #########################################################################

  ; --------------------------------------------------
  ; This procedure was written by Raymond Filiatreault
  ; --------------------------------------------------

    .386
    .model flat, stdcall  ; 32 bit memory model
    option casemap :none  ; case sensitive

      include \masm32\include\Mix.inc

    .code

; #########################################################################

MixMul proc public MInt1:DWORD, MInt2:DWORD
        
    ; -------------------------------------------------------------
    ; Multiplies a Mixed Integer DWORD by another Mixed Integer DWORD
    ;
    ; eax, ecx, edx used
    ;
    ; MInt1:    A Mixed Integer DWORD.
    ;
    ; MInt2:    A Mixed Integer DWORD.
    ;
    ; EXAMPLE:
    ;
    ;       invoke MixMul, var1, var2 
    ;
    ; The result is returned in eax in a Mixed Integer format. If the
    ; resulting absolute value of the integer portion would exceed 7FFFh
    ; the returned value would be 80000000h (i.e. -0).
    ;
    ; If any of the two parameters is 80000000h, the returned value
    ; would also be 80000000h.
    ;
    ; N.B.  The two parameters must be in the Mixed Integer format.
    ; If the multiplication of a Mixed Integer format by a regular integer
    ; is required, the regular integer must first be converted to a Mixed
    ; Integer format to use this procedure.
    ;
    ; -------------------------------------------------------------

    mov eax,MInt1
    cmp eax,ERROR
    jnz @F
    ret

  @@:
    mov ecx,MInt2
    cmp ecx,ERROR
    jnz @F
    mov eax,ecx
    ret

  @@:    
    imul ecx
    or  edx,edx
    js  @F

;result is positive

    test edx,0FFFF8000h
    jnz mulerror
    jmp mulend

;result is negative

  @@:
    mov ecx,edx
    not ecx
    test ecx,0FFFF8000h
    jnz mulerror
mulend:
    shr eax,16              ;shift the most significant 16 bits of the
                            ;fractional portion to the lower 16 bits 
    shl edx,16              ;put the integer portion in the upper 16 bits
    add eax,edx             ;combine the two
    ret

mulerror:
    mov eax,ERROR
    ret

MixMul endp

; #########################################################################

end
