; #########################################################################

  ; --------------------------------------------------
  ; This procedure was written by Raymond Filiatreault
  ; --------------------------------------------------

    .386
    .model flat, stdcall  ; 32 bit memory model
    option casemap :none  ; case sensitive

      include \masm32\include\Mix.inc

      includelib \masm32\lib\Mix.lib

    .code

; #########################################################################

MixSin proc public MInt:DWORD
        
    ; --------------------------------------------------------------
    ; The parameter is first reduced to the 0-360 degree range and then
    ; to a 0-90 degree range for the computation.
    ;    
    ; Computes the sine of an angle per the following Taylor series if
    ; the reduced angle is in the 0-45 degree range:
    ;
    ;           sin(x) = x - x^3/3! + x^5/5! ....
    ;
    ; If the angle is in the 45-90 degree range, the MixCos procedure is
    ; invoked with the complementary angle for improved speed.
    ;
    ; eax, ecx, edx used
    ;
    ; MInt:    The angle in degrees in a Mixed Integer format
    ;
    ; EXAMPLE:
    ;
    ;       invoke MixSin, var1 
    ;
    ; The result is returned in eax in a Mixed Integer format. If the
    ; parameter is the error code 80000000h, it is returned unchanged.
    ;
    ; The angle provided by the parameter can be positive or negative.
    ; Angles having absolute values above 360 degrees are also allowed.
    ;
    ; -------------------------------------------------------------

    LOCAL   summation   :DWORD
    LOCAL   netangle    :DWORD
    LOCAL   workangle   :DWORD

    mov eax,MInt
    .if eax == ERROR
        ret
    .endif

    push ebx
    push esi
    push edi

    mov edi,eax         ;keep it there for the sign
    or  eax,eax
    jns @F
    neg eax             ;make it positive
  @@:
    mov ebx,eax
    and ebx,0ffffh      ;keep only the fraction part in the lower 16 bits
    shr eax,16          ;keep only the integer part in eax
    xor edx,edx
    mov ecx,360
    div ecx             ;get the modulo 360 in edx
    mov eax,edx
    shl eax,16          ;move it back as the integer part
    add eax,ebx         ;add back the fractional part

    or  edi,edi
    jns @F
    neg eax
    add eax,360 shl 16        ;make it a positive angle 0-360
  @@:
    mov netangle,eax          ;angle in the 0-360 degree range

    .if eax > 180 shl 16      ;i.e. >180 deg
        sub eax,180 shl 16
    .endif

    .if eax > 90 shl 16       ;i.e. >90 deg
        sub eax,180 shl 16
        neg eax
    .endif

    .if eax > 45 shl 16       ;i.e. >45 deg
        sub eax,90 shl 16
        neg eax               ;complementary angle
        invoke MixCos,eax     ;i.e. cos(90-angle) = sin(angle)
        jmp cleanup
    .endif

;convert degrees to radians

    mov ecx,MixPI
    mul ecx
    mov ecx,180 shl 16
    div ecx
    mov workangle,eax         ;angle converted in radians
    mov summation,eax
    
    mov edi,1                 ;for current factorial starting at 1!
    mov esi,edi               ;for current number starting at 1
    xor ebx,ebx               ;for sign flag starting at +

  .while eax != 0

      xor ebx,80000000h       ;change sign flag
      
      mov eax,edi
      inc esi
      mul esi                 ;get next factorial
      inc esi
      mul esi                 ;   id
      mov edi,eax             ;store back as current factorial
      
      mov eax,workangle
      mov ecx,esi
    @@:
      dec ecx
      jz  @F
      push ecx
      invoke MixMul,eax,workangle     ;(angle)^esi
      pop ecx
      jmp @B
  
    @@:
      xor edx,edx
      div edi
      or  ebx,ebx
      jns @F
      neg eax
    @@:
      add summation,eax

  .endw

    mov eax,summation
cleanup:
    cmp netangle,180 shl 16   ;if angle =< 180, sin is positive
    jbe eXit
    neg eax

eXit:
    pop edi
    pop esi
    pop ebx    
    ret

MixSin endp

; #########################################################################

end

