; #########################################################################

  ; --------------------------------------------------
  ; This procedure was written by Raymond Filiatreault
  ; --------------------------------------------------

    .386
    .model flat, stdcall  ; 32 bit memory model
    option casemap :none  ; case sensitive

      include \masm32\include\Mix.inc

    .code

; #########################################################################

MixSub proc public MInt1:DWORD, MInt2:DWORD
        
    ; -------------------------------------------------------------
    ; Subtracts a Mixed Integer DWORD from another Mixed Integer DWORD
    ;
    ; eax used
    ;
    ; MInt1:    A Mixed Integer DWORD.
    ;
    ; MInt2:    A Mixed Integer DWORD to be subtracted from MInt1.
    ;
    ; EXAMPLE:
    ;
    ;       invoke MixSub, var1, var2 
    ;
    ; The result is returned in eax in a Mixed Integer format. If the
    ; resulting absolute value of the integer portion exceeds 7FFFh, the
    ; returned value would then be 80000000h (i.e. -0).
    ;
    ; If any of the two parameters is 80000000h, the returned value
    ; would also be 80000000h.
    ;
    ; N.B.  The two parameters must be in the Mixed Integer format.
    ; If the subtraction of a Mixed Integer format and a regular integer
    ; is required, the regular integer must first be converted to a Mixed
    ; Integer format.
    ;
    ; -------------------------------------------------------------

    mov eax,MInt1
    cmp eax,ERROR
    jnz @F
    ret
  @@:
    mov eax,MInt2
    cmp eax,ERROR
    jnz @F
    ret
  @@:
    xor eax,MInt1
    js @F              ;jump if both parameters have different signs

;same sign parameters cannot cause an overflow by subtraction

    mov eax,MInt1
    sub eax,MInt2
    ret

;the two parameters have different signs

  @@:
    mov eax,MInt1
    or  eax,eax
    js  @F              ;jump if first parameter is negative

;first parameter is positive, the other is negative and the result
;should be positive
    
    mov eax,MInt1
    sub eax,MInt2
    js  suberror
    ret

;first parameter is negative, the other is positive and the result
;should be negative
    
  @@:
    mov eax,MInt1
    sub eax,MInt2
    jns suberror
    ret

suberror:
    mov eax,ERROR
    ret
    
MixSub endp

; #########################################################################

end
