; #########################################################################

  ; --------------------------------------------------
  ; This procedure was written by Raymond Filiatreault
  ; --------------------------------------------------

    .386
    .model flat, stdcall  ; 32 bit memory model
    option casemap :none  ; case sensitive

      include \masm32\include\Mix.inc

      includelib \masm32\lib\Mix.lib

    .code

; #########################################################################

MixTanh proc public MInt:DWORD
        
    ; -------------------------------------------------------------
    ; Computes the hyperbolic tangent of a number per the following
    ; relationship:
    ;                     sinh(x)
    ;           tanh(x) = -------
    ;                     cosh(x)
    ;
    ; eax, ecx, edx used
    ;
    ; MInt:    The number in a Mixed Integer format
    ;
    ; EXAMPLE:
    ;
    ;       invoke MixTanh, var1 
    ;
    ; The result is returned in eax in a Mixed Integer format. If the
    ; parameter is the error code 80000000h, it is returned unchanged.
    ;
    ; The parameter can be any positive or negative number in the Mixed
    ; Integer format.
    ;
    ; Note, however, that the value of tanh(x) ranges from -1 to +1.
    ; It approaches an absolute value of 1 quite rapidly, being at
    ; 0.99999 with a parameter of 6. All parameters having an absolute
    ; value higher than 6 will return a value of +1 or -1 based on the
    ; sign of the parameter.
    ;
    ; -------------------------------------------------------------

    mov eax,MInt
    .if eax == ERROR
        ret
    .endif

    or  eax,eax
    jns @F
    neg eax
@@:
    .if eax > 6 shl 16        ;higher than 6.0?
        mov eax,1 shl 16      ;no computing necessary
        test MInt,80000000h
        jz  @F
        neg eax
      @@:
        ret
    .endif

    invoke MixCosh,MInt
    push eax
    invoke MixSinh,MInt
    pop ecx
    invoke MixDiv,eax,ecx
    
    ret

MixTanh endp

; #########################################################################

end

