; #########################################################################

  ; --------------------------------------------------
  ; This procedure was written by Raymond Filiatreault
  ; --------------------------------------------------

    .386
    .model flat, stdcall  ; 32 bit memory model
    option casemap :none  ; case sensitive

      include \masm32\include\Mix.inc

    .code

; #########################################################################

MixToA proc public MInt:DWORD, lpBuf:DWORD
	
    ; -------------------------------------------------------------
    ; Returns the decimal representation of a Mixed Integer value.
    ;
    ; eax, ecx, edx used
    ;
    ; MInt:     A Mixed Integer DWORD
    ;
    ; lpBuf:    Address of buffer where the decimal representation of the
    ;           MInt parameter will be returned with a terminating "0".
    ;
    ; If the value of the MInt parameter is 80000000h (i.e. -0), that
    ; value is returned in eax and an empty szstring is returned to the
    ; lpBuf. Otherwise, eax will have the last digit.
    ; 
    ; N.B.  The highest number of characters returned to the buffer is 13,
    ;       at most 5 for the integer portion, the decimal point, at most
    ;       5 for the fractional portion (the last one being rounded, more
    ;       decimal digits not being significant), and the terminating 0.
    ;       A "-" sign may also precede the number if it is negative.
    ;       If the fractional part is 0, only one 0 digit is returned 
    ;       for the fractional part after the decimal point.
    ;       If the MInt parameter is 0, only a 0 digit is returned without 
    ;       any decimal point.
    ;
    ; -------------------------------------------------------------

    push ebx
    push edi

    mov edi,lpBuf
    mov eax,MInt
    cmp eax,ERROR
    jnz @F
    stosb               ;store a terminating 0 as first character in buffer
    jmp eXit

  @@:
    or  eax,eax
    jnz @F
    jmp lastdec1        ;no use calculating if the input is 0

  @@:
    jns @F
    mov byte ptr [edi],"-"
    inc edi
    neg eax

  @@:
    shr eax,16
    mov ebx,eax
    xor edx,edx
    mov ecx,1000
    div ecx
    or  eax,eax
    mov cx,10
    jz  nexttwo
    div cl
    add ax,3030h
    cmp al,"0"
    jnz @F
    mov al,ah
    stosb
    jz  nexttwo
  @@:
    stosw

nexttwo:
    mov eax,edx
    xor edx,edx
    div ecx
    or  eax,eax
    jnz @F
    cmp ebx,1000
    jb  lastone
  @@:
    div cl
    add ax,3030h
    cmp al,"0"
    jnz @F
    cmp ebx,999
    ja  @F
    mov al,ah
    stosb
    jb  lastone
  @@:
    stosw

lastone:
    mov al,dl
    add al,"0"
    mov ah,"."
    stosw

    mov edx,MInt
    or  edx,edx
    jns @F
    neg edx
  @@:
    shl edx,16
    or  edx,edx                 ;is the fractional part = 0?
    jz  lastdec

    mov ebx,50000000
    xor eax,eax
    mov ecx,16
frac1:
    shl edx,1
    jnc frac2
    add eax,ebx
frac2:
    shr ebx,1
    dec cx
    jnz frac1

    mov ebx,1000
    div ebx
    
    .if edx > 500
        inc eax
    .endif

    xor edx,edx
    mov ebx,1000
    div ebx
    mov ebx,10
    div bl
    add ax,3030h
    stosw
    mov eax,edx
    xor edx,edx
    div ebx
    div bl
    add ax,3030h
    stosw
lastdec:
    mov eax,edx
lastdec1:
    add al,30h
    stosw

eXit:    
    pop edi
    pop ebx
    ret
    
MixToA endp

; #########################################################################

end
