; #########################################################################
;
;             Prepared by Raymond Filiatreault, May 2002
;
;  MIXTEST is a program to test all the trigonometric, hyperbolic,
;  logarithmic and exponential functions of the Mix.lib library, and
;  compare the results with those obtained with the FPU with the same
;  converted input.
;
;  Although the reciprocal functions, such as cotan(x) being the reciprocal
;  of tan(x), are not part of the Mix.lib library, they are included to
;  show how they can be derived and which precautions must be taken. 
;
;  The results are displayed in a 'read only' multiline edit window, the
;  content of which can be scrolled at will, and saved at any time.
;
; #########################################################################

      .386
      .model flat, stdcall
      option casemap :none   ; case sensitive

; #########################################################################

      include \masm32\include\windows.inc
      include \masm32\include\user32.inc
      include \masm32\include\kernel32.inc
      include \masm32\include\comdlg32.inc
      include \masm32\include\mix.inc
      include \masm32\include\masm32.inc
      include \masm32\include\comctl32.inc

      includelib \masm32\lib\user32.lib
      includelib \masm32\lib\kernel32.lib
      includelib \masm32\lib\comdlg32.lib
      includelib \masm32\lib\mix.lib
      includelib \masm32\lib\masm32.lib
      includelib \masm32\lib\comctl32.lib

      include MixtestD.asm        ;data file and mundane subroutines

; #########################################################################

    ;=================
    ; Local prototypes
    ;=================
    WndProc PROTO :DWORD,:DWORD,:DWORD,:DWORD
    
; #########################################################################

    .code

start:

        invoke GetModuleHandle, NULL
        mov hInstance, eax
        
        invoke DialogBoxParam,hInstance,700,NULL,ADDR WndProc,NULL

        invoke ExitProcess,eax

; #########################################################################

WndProc proc hWin   :DWORD,
             uMsg   :DWORD,
             wParam :DWORD,
             lParam :DWORD

      .if uMsg == WM_INITDIALOG
        m2m     hDlg,hWin
        invoke  CheckRadioButton,hWin,710,736,710
        invoke  SendDlgItemMessage,hWin,766,WM_ENABLE,FALSE,0
        invoke  LocalAlloc,LMEM_FIXED,4000
        mov     hBuf,eax
        invoke  LocalSize,eax
        mov     bufsize,eax
        mov     available,eax
        invoke  InitCommonControls
        return  TRUE

      .elseif uMsg == WM_COMMAND
            mov     eax,wParam
            and     eax,0ffffh
            .if     eax > 709 && eax < 737      ;radio button clicked
                sub     eax,710
                mov     fselected,eax
                shl     eax,2
                mov     eax,param1adds[eax]
                invoke  SetDlgItemText,hWin,760,eax
                .if     fselected > 5 && fselected < 12
                    invoke  SendDlgItemMessage,hWin,766,WM_ENABLE,TRUE,0
                    invoke  SetDlgItemText,hWin,761,ADDR param2txt1
                .else
                    invoke  SendDlgItemMessage,hWin,766,WM_ENABLE,FALSE,0
                    invoke  SetDlgItemText,hWin,761,ADDR nulltxt
                    invoke  SetDlgItemText,hWin,766,ADDR nulltxt
                .endif
            .elseif eax == 770          ;CALCULATE clicked
                call    getparam1
                .if     eax == ERROR
                    invoke  MessageBox,0,ADDR param1err,0,MB_OK or MB_ICONHAND
                    return  TRUE
                .endif
                
                .if     fselected > 5 && fselected < 12
                    call    getparam2
                    .if     eax == ERROR
                        invoke  MessageBox,0,ADDR param2err,0,MB_OK or MB_ICONHAND
                        return TRUE
                    .endif
                .endif
                
                .if     fselected < 6
                    call    quadrant        ;convert input to 0-360 degrees
                .endif
                
                mov     eax,fselected
                shl     eax,2
                call    functionx[eax]
                return  TRUE
                
            .elseif eax == 771          ;SAVE AS... clicked
                call    savedata
                
            .elseif eax == 772          ;EXIT clicked
                invoke  LocalFree,hBuf
                invoke  EndDialog,hWin,0
            .endif
        
      .elseif uMsg == WM_CLOSE
        invoke  LocalFree,hBuf
        invoke  EndDialog,hWin,0

      .endif

      return FALSE

WndProc endp

; ########################################################################

sinx:

    fninit
    fild    param1          ;load the angle parameter (Mixed Integer)
    fidiv   c64k            ;convert to its proper binary fraction
    fldpi                   ;load value of PI
    fmul
    fidiv   w180            ;convert degrees to radians
    fsin
    fimul   c64k            ;convert to a Mixed Integer
    fistp   fpuresult       ;transfer result to memory

    invoke  MixSin,param1
    mov     mixresultA,eax

    xor     ecx,ecx
    .if     quadrantx == 1 || quadrantx == 2
        inc     ecx
    .endif

    invoke  MixArcsin,eax,ecx
    mov     mixresultB,eax
    
    call    showres
    
    ret

;-----------------------------

cosx:

    fninit
    fild    param1          ;load the angle parameter (Mixed Integer)
    fidiv   c64k            ;convert to its proper binary fraction
    fldpi                   ;load value of PI
    fmul
    fidiv   w180            ;convert degrees to radians
    fcos
    fimul   c64k            ;convert to a Mixed Integer
    fistp   fpuresult       ;transfer result to memory

    invoke  MixCos,param1
    mov     mixresultA,eax

    xor     ecx,ecx
    .if     quadrantx > 1
        inc     ecx
    .endif

    invoke  MixArccos,eax,ecx
    mov     mixresultB,eax
    
    call    showres
    
    ret

;-----------------------------

tanx:

    .if     param1 == 90 shl 16 || param1 == 270 shl 16
        mov     eax,7fffffffh
        jmp     tanx1
    .endif

    fninit
    fild    param1          ;load the angle parameter (Mixed Integer)
    fidiv   c64k            ;convert to its proper binary fraction
    fldpi                   ;load value of PI
    fmul
    fidiv   w180            ;convert degrees to radians
    fptan
    fistp   doubled         ;get rid of the 1
    fimul   c64k            ;convert to a Mixed Integer
    fistp   doubled         ;transfer result to memory
    lea     eax,doubled
    mov     ecx,[eax+4]
    mov     eax,[eax]
    
    .if     ecx == -1       ;result should be negative
        or      eax,eax
        js      @F
        mov     eax,-7fffffffh
      @@:
    .elseif ecx == 0       ;result should be positive
        or      eax,eax
        jns     @F
        mov     eax,7fffffffh
      @@:
    .else                   ;result overflows limits
        mov     eax,7fffffffh
        or      ecx,ecx
        jns     @F
        neg eax
     @@:
    .endif

tanx1:
    mov     fpuresult,eax

    invoke  MixTan,param1
    mov     mixresultA,eax

    mov     ecx,1 shl 16
    .if     quadrantx == 1 || quadrantx == 2
        neg     ecx
        neg     eax
    .endif

    invoke  MixArctan,eax,ecx
    mov     mixresultB,eax
    
    call    showres
    
    ret

;-----------------------------

cotx:

    .if     param1 == 0 || param1 == 180 shl 16
        mov     eax,7fffffffh
        jmp     cotx1
    .endif

    fninit
    fild    param1          ;load the angle parameter (Mixed Integer)
    fidiv   c64k            ;convert to its proper binary fraction
    fldpi                   ;load value of PI
    fmul
    fidiv   w180            ;convert degrees to radians
    fptan
    fdivr
    fimul   c64k            ;convert to a Mixed Integer
    fistp   doubled         ;transfer result to memory
    lea     eax,doubled
    mov     ecx,[eax+4]
    mov     eax,[eax]
    
    .if     ecx == -1       ;result should be negative
        or      eax,eax
        js      @F
        mov     eax,-7fffffffh
      @@:
    .elseif ecx == 0       ;result should be positive
        or      eax,eax
        jns     @F
        mov     eax,7fffffffh
      @@:
    .else                   ;result overflows limits
        mov     eax,7fffffffh
        or      ecx,ecx
        jns     @F
        neg eax
     @@:
    .endif

cotx1:
    mov     fpuresult,eax

    invoke  MixTan,param1
    mov     edx,eax
    or      edx,edx
    jns     @F
    neg edx
  @@:
    .if     edx < 3
        mov     eax,7fffffffh
        test    quadrantx,1
        jz      @F
        neg     eax
      @@:
    .else
        invoke  MixDiv,10000h,eax
    .endif
    mov     mixresultA,eax

    invoke  MixDiv,10000h,eax
    .if     eax == ERROR
        dec     eax
        mov     edx,mixresultA
        or      edx,edx
        jns     @F
        neg     eax
      @@:
    .endif

    mov     ecx,1 shl 16
    .if     quadrantx == 1 || quadrantx == 2
        neg     ecx
        neg     eax
    .endif

    invoke  MixArctan,eax,ecx
    mov     mixresultB,eax
    
    call    showres
    
    ret

;-----------------------------

secx:

    .if     param1 == 90 shl 16 || param1 == 270 shl 16
        mov     eax,7fffffffh
        jmp     secx1
    .endif

    fninit
    fild    param1          ;load the angle parameter (Mixed Integer)
    fidiv   c64k            ;convert to its proper binary fraction
    fldpi                   ;load value of PI
    fmul
    fidiv   w180            ;convert degrees to radians
    fcos
    fld1
    fdivr
    fimul   c64k            ;convert to a Mixed Integer
    fistp   doubled         ;transfer result to memory
    lea     eax,doubled
    mov     ecx,[eax+4]
    mov     eax,[eax]
    
    .if     ecx == -1       ;result should be negative
        or      eax,eax
        js      @F
        mov     eax,-7fffffffh
      @@:
    .elseif ecx == 0       ;result should be positive
        or      eax,eax
        jns     @F
        mov     eax,7fffffffh
      @@:
    .else                   ;result overflows limits
        mov     eax,7fffffffh
        or      ecx,ecx
        jns     @F
        neg eax
     @@:
    .endif

secx1:
    mov     fpuresult,eax

    invoke  MixCos,param1
    invoke  MixDiv,10000h,eax
    .if     eax == ERROR
        dec     eax
        .if     quadrantx == 1 || quadrantx == 2
            neg     eax
        .endif
    .endif
    mov     mixresultA,eax

    invoke  MixDiv,10000h,eax
    .if     eax == ERROR
        dec     eax
        .if     quadrantx == 1 || quadrantx == 2
            neg     eax
        .endif
    .endif

    xor     ecx,ecx
    .if     quadrantx > 1
        inc     ecx
    .endif

    invoke  MixArccos,eax,ecx
    mov     mixresultB,eax
    
    call    showres
    
    ret

;-----------------------------

cscx:

    .if     param1 == 0 || param1 == 180 shl 16
        mov     eax,7fffffffh
        jmp     cscx1
    .endif

    fninit
    fild    param1          ;load the angle parameter (Mixed Integer)
    fidiv   c64k            ;convert to its proper binary fraction
    fldpi                   ;load value of PI
    fmul
    fidiv   w180            ;convert degrees to radians
    fsin
    fld1
    fdivr
    fimul   c64k            ;convert to a Mixed Integer
    fistp   doubled         ;transfer result to memory
    lea     eax,doubled
    mov     ecx,[eax+4]
    mov     eax,[eax]
    
    .if     ecx == -1       ;result should be negative
        or      eax,eax
        js      @F
        mov     eax,-7fffffffh
      @@:
    .elseif ecx == 0       ;result should be positive
        or      eax,eax
        jns     @F
        mov     eax,7fffffffh
      @@:
    .else                   ;result overflows limits
        mov     eax,7fffffffh
        or      ecx,ecx
        jns     @F
        neg eax
     @@:
    .endif

cscx1:
    mov     fpuresult,eax

    invoke  MixSin,param1
    invoke  MixDiv,10000h,eax
    .if     eax == ERROR
        dec     eax
        .if     quadrantx > 1
            neg     eax
        .endif
    .endif
    mov     mixresultA,eax

    invoke  MixDiv,10000h,eax
    .if     eax == ERROR
        dec     eax
        .if     quadrantx > 1
            neg     eax
        .endif
    .endif

    xor     ecx,ecx    
    .if     quadrantx == 1 || quadrantx == 2
        inc     ecx
    .endif

    invoke  MixArcsin,eax,ecx
    mov     mixresultB,eax
    
    call    showres
    
    ret

;-----------------------------

asinx:

    mov     eax,param1
    or      eax,eax
    jns     @F
    neg     eax
  @@:
    .if     eax == 1 shl 16
        mov     eax,90 shl 16
        .if     param1 == -1 shl 16
            mov     eax,270 shl 16
        .endif
        jmp     asinx1      ;to avoid division by 0
    .endif

    fninit
    fild    param1          ;load the input
    fidiv   c64k            ;convert to its proper binary fraction
    fld     st(0)           ;copy it
    fmul    st,st(0)        ;sin^2
    fld1
    fsubr                   ;->cos^2
    fsqrt
    .if     param2 == -1 shl 16
        fchs
    .endif
    fpatan
    fimul   w180
    fldpi
    fdiv                    ;back to degrees
    fimul   c64k            ;convert to a Mixed Integer
    fistp   fpuresult

    mov     eax,fpuresult
    or      eax,eax
    jns     asinx1
    add     eax,360 shl 16

asinx1:
    mov     fpuresult,eax

    xor     eax,eax
    .if     param2 == -1 shl 16
        mov     eax,-1 shl 16
    .endif
    invoke  MixArcsin,param1,eax
    mov     mixresultA,eax

    invoke  MixSin,eax
    mov     mixresultB,eax
    
    call    showres

    ret

;-----------------------------

acosx:

    .if     param1 == 0
        mov     eax,90 shl 16
        .if     param2 == -1 shl 16
            mov     eax,270 shl 16
        .endif
        jmp     acosx1      ;to avoid division by 0
    .endif

    fninit
    fild    param1          ;load the input
    fidiv   c64k            ;convert to its proper binary fraction
    fld     st(0)           ;copy it
    fmul    st(0),st(0)     ;cos^2
    fld1
    fsubr                   ;->sin^2
    fsqrt
    .if     param2 == -1 shl 16
        fchs
    .endif
    fxch
    fpatan
    fimul   w180
    fldpi
    fdiv                    ;back to degrees
    fimul   c64k            ;convert to a Mixed Integer
    fistp   fpuresult

    mov     eax,fpuresult
    or      eax,eax
    jns     acosx1
    add     eax,360 shl 16

acosx1:
    mov     fpuresult,eax

    xor     eax,eax
    .if     param2 == -1 shl 16
        mov     eax,-1 shl 16
    .endif
    invoke  MixArccos,param1,eax
    mov     mixresultA,eax

    invoke  MixCos,eax
    mov     mixresultB,eax
    
    call    showres

    ret

;-----------------------------

atanx:

    fninit
    fild    param1          ;load the input
    fild    param2
    fpatan
    fimul   w180
    fldpi
    fdiv                    ;back to degrees
    fimul   c64k            ;convert to a Mixed Integer
    fistp   fpuresult

    .if     fpuresult > ERROR   ;i.e. negative
        add     fpuresult,360 shl 16
    .endif

    invoke  MixArctan,param1,param2
    mov     mixresultA,eax

    invoke  MixTan,eax
    mov     mixresultB,eax
    
    call    showres

    ret

;-----------------------------

acotx:

    push    param1
    .if     param1 == 0
        mov     eax,90 shl 16
        .if     param2 == -1 shl 16
            mov     eax,270 shl 16
        .endif
        mov     fpuresult,eax
        mov     mixresultA,eax
        jmp     acotx1          ;to avoid division by 0
    .endif

    mov     eax,param1
    mov     ecx,param2
    mov     edx,ecx
    xor     edx,eax
    jns     @F
    neg     eax
    neg     ecx
    mov     param1,eax
    mov     param2,ecx     
  @@:
    fninit
    fild    param2
    fild    param1          ;load the input
    fpatan
    fimul   w180
    fldpi
    fdiv                    ;back to degrees
    fimul   c64k            ;convert to a Mixed Integer
    fistp   fpuresult

    mov     eax,fpuresult
    or      eax,eax
    jns     @F
    add     eax,360 shl 16
  @@:
    mov     fpuresult,eax

    invoke  MixDiv,10000h,param1

    invoke  MixArctan,param2,param1
    mov     mixresultA,eax

acotx1:
    .if     mixresultA == 90 shl 16 || mixresultA == 270 shl 16
        xor     eax,eax
        jmp     acotx2
    .endif
    invoke  MixTan,eax
    invoke  MixDiv,10000h,eax       ;get reciprocal
    .if     eax == ERROR
        dec     eax
        .if     mixresultA < 180 shl 16
            .if     mixresultA > 90 shl 16
                neg     eax
            .endif
        .else
            .if     mixresultA > 270 shl 16
                neg     eax
            .endif
        .endif
    .endif
acotx2:
    mov     mixresultB,eax

    pop     param1

    call    showres

    ret

;-----------------------------

asecx:

    fninit
    fild    param1          ;load the input
    fidiv   c64k            ;convert to its proper binary fraction
    fld1
    fdivr                   ;get reciprocal
    fld     st(0)           ;copy it
    fmul    st(0),st(0)     ;cos^2
    fld1
    fsubr                   ;->sin^2
    fsqrt
    .if     param2 == -1 shl 16
        fchs
    .endif
    fxch
    fpatan
    fimul   w180
    fldpi
    fdiv                    ;back to degrees
    fimul   c64k            ;convert to a Mixed Integer
    fistp   fpuresult

    mov     eax,fpuresult
    or      eax,eax
    jns     @F
    add     eax,360 shl 16
  @@:
    mov     fpuresult,eax

    invoke  MixDiv,10000h,param1    ;get reciprocal
    xor     ecx,ecx
    .if     param2 == -1 shl 16
        mov     ecx,-1 shl 16
    .endif
    invoke  MixArccos,eax,ecx
    mov     mixresultA,eax

    invoke  MixCos,eax
    invoke  MixDiv,10000h,eax       ;get reciprocal
    .if     eax == ERROR
        dec     eax
        .if     mixresultA > 90 shl 16 && mixresultA < 270 shl 16
            neg     eax
        .endif
    .endif
    mov     mixresultB,eax

    call    showres

    ret

;-----------------------------

acscx:

    mov     eax,param1
    or      eax,eax
    jns     @F
    neg     eax
  @@:
    .if     eax == 1 shl 16
        mov     eax,90 shl 16
        .if param1 == -1 shl 16
            mov     eax,270 shl 16
        .endif
        jmp     acscx1          ;to avoid division by 0
    .endif

    fninit
    fild    param1          ;load the input
    fidiv   c64k            ;convert to its proper binary fraction
    fld1
    fdivr                   ;get reciprocal
    fld     st(0)           ;copy it
    fmul    st(0),st(0)     ;sin^2
    fld1
    fsubr                   ;->cos^2
    fsqrt
    .if     param2 == -1 shl 16
        fchs
    .endif
    fpatan
    fimul   w180
    fldpi
    fdiv                    ;back to degrees
    fimul   c64k            ;convert to a Mixed Integer
    fistp   fpuresult

    mov     eax,fpuresult
    or      eax,eax
    jns     acscx1
    add     eax,360 shl 16

acscx1:
    mov     fpuresult,eax

    invoke  MixDiv,10000h,param1    ;get reciprocal
    xor     ecx,ecx
    .if     param2 == -1 shl 16
        mov     ecx,-1 shl 16
    .endif
    invoke  MixArcsin,eax,ecx
    mov     mixresultA,eax

    invoke  MixSin,eax
    invoke  MixDiv,10000h,eax       ;get reciprocal
    .if     eax == ERROR
        dec     eax
        .if     mixresultA > 180 shl 16
            neg     eax
        .endif
    .endif
    mov     mixresultB,eax

    call    showres

    ret

;-----------------------------

sinhx:

    finit
    call    fpusinh
    fistp   fpuresult

    invoke  MixSinh,param1
    mov     mixresultA,eax

    invoke  MixArcsinh,eax
    mov     mixresultB,eax

    call    showres

    ret

;-----------------------------

coshx:

    finit
    call    fpucosh
    fistp   fpuresult

    invoke  MixCosh,param1
    mov     mixresultA,eax

    mov     ecx,param1
    or      ecx,ecx
    jns     @F
    neg     eax
  @@:
    invoke  MixArccosh,eax
    mov     mixresultB,eax

    call    showres

    ret

;-----------------------------

tanhx:

    finit
    call    fpusinh
    call    fpucosh
    fdiv
    fimul   c64k
    fistp   fpuresult

    invoke  MixTanh,param1
    mov     mixresultA,eax

    invoke  MixArctanh,eax
    mov     mixresultB,eax

    call    showres

    ret

;-----------------------------

cothx:

    .if     param1 == 0
        mov     fpuresult,7fffffffh
        jmp     cothx1
    .endif

    finit
    call    fpucosh
    call    fpusinh
    fdiv
    fimul   c64k
    fistp   doubled         ;transfer result to memory
    lea     eax,doubled
    mov     ecx,[eax+4]
    mov     eax,[eax]
    
    .if     ecx == -1       ;result should be negative
        or      eax,eax
        js      @F
        mov     eax,-7fffffffh
      @@:
    .elseif ecx == 0       ;result should be positive
        or      eax,eax
        jns     @F
        mov     eax,7fffffffh
      @@:
    .else                   ;result overflows limits
        mov     eax,7fffffffh
        or      ecx,ecx
        jns     @F
        neg eax
     @@:
    .endif
    mov     fpuresult,eax

cothx1:
    invoke  MixTanh,param1
    push    eax
    invoke  MixDiv,10000h,eax
    pop     ecx
    .if     eax == ERROR
        dec     eax
        or      ecx,ecx
        jns     @F
        neg     eax
      @@:
    .endif
    mov     mixresultA,eax

    invoke  MixDiv,10000h,eax
    invoke  MixArctanh,eax
    mov     mixresultB,eax

    call    showres

    ret

;-----------------------------

sechx:

    finit
    call    fpucosh
    fidivr  c64k
    fimul   c64k
    fistp   fpuresult

    invoke  MixCosh,param1
    Invoke  MixDiv,10000h,eax
    mov     mixresultA,eax

    Invoke  MixDiv,10000h,eax
    .if     eax == ERROR
        dec     eax
    .endif
    .if     param1 > ERROR      ;i.e. negative
        neg     eax
    .endif
    invoke  MixArccosh,eax
    mov     mixresultB,eax

    call    showres

    ret

;-----------------------------

cschx:

    .if     param1 == 0
        mov     fpuresult,7fffffffh
        jmp     cschx1
    .endif

    finit
    call    fpusinh
    fidivr  c64k
    fimul   c64k
    fistp   doubled         ;transfer result to memory
    lea     eax,doubled
    mov     ecx,[eax+4]
    mov     eax,[eax]
    
    .if     ecx == -1       ;result should be negative
        or      eax,eax
        js      @F
        mov     eax,-7fffffffh
      @@:
    .elseif ecx == 0       ;result should be positive
        or      eax,eax
        jns     @F
        mov     eax,7fffffffh
      @@:
    .else                   ;result overflows limits
        mov     eax,7fffffffh
        or      ecx,ecx
        jns     @F
        neg eax
     @@:
    .endif
    mov     fpuresult,eax

cschx1:
    invoke  MixSinh,param1
    Invoke  MixDiv,10000h,eax
    .if     eax == ERROR
        dec     eax
        .if     param1 > ERROR      ;i.e. negative
            neg     eax
        .endif
    .endif
    mov     mixresultA,eax

    Invoke  MixDiv,10000h,eax
    .if     eax == ERROR
        dec     eax
        .if     mixresultA > ERROR      ;i.e. negative
            neg     eax
        .endif
    .endif
    invoke  MixArcsinh,eax
    mov     mixresultB,eax

    call    showres

    ret

;-----------------------------

asinhx:

    finit
    call    fpuasinh
    fistp   fpuresult

    invoke  MixArcsinh,param1
    mov     mixresultA,eax

    invoke  MixSinh,eax
    mov     mixresultB,eax

    call    showres

    ret

;-----------------------------

acoshx:

    finit
    call    fpuacosh
    fistp   fpuresult
    .if     param1 > ERROR      ;i.e. negative
        neg     fpuresult
    .endif

    invoke  MixArccosh,param1
    mov     mixresultA,eax

    invoke  MixCosh,eax
    mov     mixresultB,eax

    call    showres

    ret

;-----------------------------

atanhx:

    finit
    call    fpuatanh
    fistp   fpuresult

    invoke  MixArctanh,param1
    mov     mixresultA,eax

    invoke  MixTanh,eax
    mov     mixresultB,eax

    call    showres

    ret

;-----------------------------

acothx:

    finit
    invoke  MixDiv,10000h,param1
    push    eax
    push    param1
    mov     param1,eax
    call    fpuatanh
    fistp   fpuresult
    pop     param1
    pop     eax

    invoke  MixArctanh,eax
    mov     mixresultA,eax

    invoke  MixTanh,eax
    invoke  MixDiv,10000h,eax
    mov     mixresultB,eax

    call    showres

    ret

;-----------------------------

asechx:

    finit
    invoke  MixDiv,10000h,param1
    push    eax
    push    param1
    mov     param1,eax
    call    fpuacosh
    fistp   fpuresult
    pop     param1
    .if     param1 > ERROR      ;i.e. negative
        neg     fpuresult
    .endif
    pop     eax

    invoke  MixArccosh,eax
    mov     mixresultA,eax

    invoke  MixCosh,eax
    invoke  MixDiv,10000h,eax
    mov     mixresultB,eax

    call    showres

    ret

;-----------------------------

acschx:

    finit
    invoke  MixDiv,10000h,param1
    push    eax
    push    param1
    mov     param1,eax
    call    fpuasinh
    fistp   fpuresult
    pop     param1
    pop     eax

    invoke  MixArcsinh,eax
    mov     mixresultA,eax

    invoke  MixSinh,eax
    invoke  MixDiv,10000h,eax
    mov     mixresultB,eax

    call    showres

    ret

;-----------------------------

lnx:

    finit
    fld1
    fild    param1
    fidiv   c64k
    fyl2x
    fldl2e
    fdiv
    fimul   c64k
    fistp   fpuresult

    invoke  MixLNx,param1
    mov     mixresultA,eax

    invoke  MixEx,eax
    mov     mixresultB,eax

    call    showres

    ret

;-----------------------------

expx:

    finit
    fild    param1
    fidiv   c64k
    call    eexponx
    fimul   c64k
    fistp   fpuresult

    invoke  MixEx,param1
    mov     mixresultA,eax

    invoke  MixLNx,eax
    mov     mixresultB,eax

    call    showres

    ret

;-----------------------------

sqrtx:

    finit
    fild    param1
    fidiv   c64k
    fsqrt
    fimul   c64k
    fistp   fpuresult

    invoke  MixSqrt,param1
    mov     mixresultA,eax

    invoke  MixMul,eax,eax
    mov     mixresultB,eax

    call    showres

    ret

;#############################

;computes e^x
;x is taken from top of stack and replaced by result

eexponx:
    
    fldl2e
    fmul
    fld     st(0)
    frndint
    fxch
    fsub    st,st(1)
    f2xm1
    fld1
    fadd
    fscale
    fxch
    fstp    st(0)
    ret

;-----------------------------

fpusinh:

    fild    param1
    fidiv   c64k
    fld     st(0)
    call    eexponx
    fxch
    fchs
    call    eexponx
    fsub
    fimul   c32k
    ret

;-----------------------------

fpuasinh:

    fld1
    fild    param1
    fidiv   c64k
    fld     st(0)
    fmul    st,st(0)
    fld1
    fadd
    fsqrt
    fadd
    fyl2x
    fldl2e
    fdiv
    fimul   c64k
    ret

;-----------------------------

fpucosh:

    fild    param1
    fidiv   c64k
    fld     st(0)
    call    eexponx
    fxch
    fchs
    call    eexponx
    fadd
    fimul   c32k
    ret

;-----------------------------

fpuacosh:

    fld1
    fild    param1
    fidiv   c64k
    fabs
    fld     st(0)
    fmul    st,st(0)
    fld1
    fsub
    fsqrt
    fadd
    fyl2x
    fldl2e
    fdiv
    fimul   c64k
    ret

;-----------------------------

fpuatanh:

    fld1
    fild    param1
    fidiv   c64k
    fld     st(0)
    fld1
    fadd
    fxch
    fld1
    fsubr
    fdiv
    fyl2x
    fldl2e
    fdiv
    fimul   c32k
    ret

;-----------------------------

end start

